Attribute VB_Name = "modFileSystem"
' Copyright (C) 2009 - 2011 Daro Cutillas Carrillo
'
' This file is part of Smart Fpg Editor.
'
' Smart Fpg Editor is free software: you can redistribute it and/or modify
' it under the terms of the GNU General Public License as published by
' the Free Software Foundation, either version 3 of the License, or
' (at your option) any later version.
'
' Smart Fpg Editor is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with Smart Fpg Editor. If not, see <http://www.gnu.org/licenses/>.

Option Explicit

Private Const MODULE_NAME As String = "SmartFpgEditor.modFileSystem"

' Creates an absolute path to the sFileTitle file by joining it to sDir. sDir
' can end with or without a backslash "\". If the sFileTitle param is omitted
' the function returns sDir with the ending backslash.
' @param sDir the directoy of the file
' @param sFileTitle the file title (file name without the path).
' @return the path to the file resulting from joining sDir and sFileTitle
Public Function BuildPath(ByVal sDir As String, _
        Optional ByVal sFileTitle As String) As String
    
    Const PROC_NAME As String = "BuildPath"
    
    On Error GoTo EH
    
    sDir = IIf(Right(sDir, 1) = "\", sDir, sDir & "\")
    
    If Len(sFileTitle) > 0 Then
        sFileTitle = IIf(Left(sFileTitle, 1) = "\", _
                Right(sFileTitle, Len(sFileTitle) - 1), sFileTitle)
    End If
            
    BuildPath = sDir & sFileTitle
    
    Exit Function
EH:
    Err.Show MODULE_NAME, PROC_NAME
End Function

' Gets the extension of the specified file
' @param sFile the file name or path
' @return the extension of the file, i.e. the characters after the last dot of
' the filename
Public Function GetFileExtension(ByVal sFile As String) As String
    Dim i As Integer
    
    Const PROC_NAME As String = "GetFileExtension"
    
    On Error GoTo EH
    
    i = InStrRev(sFile, ".")
    If i > 0 Then
        GetFileExtension = Right(sFile, Len(sFile) - i)
    End If
    
    Exit Function
EH:
    Err.Show MODULE_NAME, PROC_NAME
End Function

' Retrieves the file title of a file path
' @param sPath the path of the file
' @param bWithoutExt if true, the returned file title will not contain the extension
' @return the file title, with or without the extension
Public Function GetFileTitle(ByVal sPath As String, _
        Optional ByVal bWithoutExt As Boolean) As String
    
    Const PROC_NAME As String = "GetFileTitle"
    
    Dim p As Integer
    Dim s As String
    
    p = InStrRev(sPath, "\")
    If p > 0 And Len(sPath) > p Then
        s = Right(sPath, Len(sPath) - p)
    End If
    
    If bWithoutExt Then
        p = InStrRev(s, ".")
        If p > 0 And Len(s) > p Then
            s = Left(s, p - 1)
        End If
    End If
    
    GetFileTitle = s
    
    Exit Function
EH:
    Err.Raise , MODULE_NAME, , , , PROC_NAME
End Function

' Gets windows temporal folder
Public Property Get TempFolderPath() As String
    Const PROC_NAME As String = "GetTempFolderPath"
    On Error GoTo EH

    If Environ("tmp") <> "" Then
        TempFolderPath = Environ("tmp")
    ElseIf Environ("temp") <> "" Then
        TempFolderPath = Environ("temp")
    Else
        TempFolderPath = CurDir()
    End If
    
    Exit Property
EH:
    Err.Raise , MODULE_NAME, , , , PROC_NAME
End Property

' Determines if we have permissions on the temp folder to write files
Public Function IsTempFolderWriteable() As Boolean
    Const PROC_NAME As String = "IsTempFolderWriteable"
    On Error GoTo EH

    If DirExists(TempFolderPath) Then
        IsTempFolderWriteable = (GetAttr(TempFolderPath) And vbReadOnly) = 0
    End If

    Exit Function
EH:
    Err.Raise , MODULE_NAME, , , , PROC_NAME
End Function

' Determines if a file exists
Public Function FileExists(sFile As String) As Boolean
    Const PROC_NAME As String = "FileExists"
    On Error GoTo EH

    FileExists = (GetAttr(sFile) And vbDirectory) = 0
    Exit Function
EH:
    'Returns False if an error occurs
End Function

' Determines if a file exists
Public Function DirExists(sDir As String) As Boolean
    Const PROC_NAME As String = "DirExists"
    On Error GoTo EH
    
    DirExists = GetAttr(sDir) And vbDirectory
    Exit Function
EH:
    'Returns False if an error occurs
End Function
